<li class="dropdown">
    <form action="{% url 'marketmanager:marketbrowser' %}" method="get" id="item_selector" class="dropdown-toggle" role="button" data-toggle="dropdown" aria-haspopup="false" aria-expanded="true">
        {% if eve_region != None %} <input type="hidden" name="region_id" value={{ eve_region.id }}>{% endif %}
            <select onchange="this.form.submit()" class="form-control" id="item_selector" name="type_id">
                {% for market_group in all_market_groups %}
                <option value="{{ market_group.id }}">
                    {{ market_group.name }}
                </option>
                {% endfor %}
            </select>
    </form>
</li>
